import os
import os.path as op
import glob
import pandas as pd
import mne
import nice
import numpy as np
import pandas as pd
import next_icm
import mne
from mne.utils import logger
from next_icm.rs.reductions import _get_rs_montage_generic
from nice import Markers
from nice.markers import (PowerSpectralDensity,
                          PowerSpectralDensitySummary,
                          PowerSpectralDensityEstimator)


def _get_markers(tmin, tmax):
    psds_params = dict(n_fft=4096, n_overlap=100, n_jobs='auto', nperseg=128)

    base_psd = PowerSpectralDensityEstimator(
        psd_method='welch', tmin=tmin, tmax=tmax, fmin=1., fmax=25.,
        psd_params=psds_params, comment='default')
    
    f_list = [
        PowerSpectralDensity(estimator=base_psd, fmin=1., fmax=4.,
                                normalize=True, comment='deltan'),
        PowerSpectralDensity(estimator=base_psd, fmin=4., fmax=8.,
                                normalize=True, comment='thetan'),
        PowerSpectralDensity(estimator=base_psd, fmin=8., fmax=12.,
                                normalize=True, comment='alphan'),

        PowerSpectralDensity(estimator=base_psd, fmin=1., fmax=25.,
                                normalize=True, comment='summary_se'),

        PowerSpectralDensitySummary(estimator=base_psd, fmin=1., fmax=25.,
                            percentile=.5, comment='summary_msf')
    ]
    fc = Markers(f_list)
    return fc

#%%
reduction_params = _get_rs_montage_generic(
    montage='neurospin/bv11', epochs_fun=np.mean, channels_fun=np.mean,
    config_params=None)

stim_conds = ['cm_3v_results', 'cm_5v_results', 'vl_3v_results', 'vl_5v_results', 'anest_results']
# There are 5 conditions:  CM_3V    |    CM_5V   |   VL_3V   |   VL_5V   | anest
for stim_cond in stim_conds: 
    inpath = '.../results/' + stim_cond
    outpath = '.../markers'
    
    subjects = os.listdir(inpath)

    df = {}
    df['marker'] = []
    df['subject'] = []
    df['value'] = []
    df['avg_value'] = []
    for s in subjects:
        logger.info('Read epochs: {}'.format(s))
        fname = glob.glob(op.join(inpath, s, '*_cut-raw_TDDon_reject-auto_epo.fif'))[0]
        epochs = mne.read_epochs(fname)
        
        logger.info('Fit markers')
            
        fc = _get_markers(tmin=None, tmax=None)
        fc.fit(epochs)
        fc = fc.reduce_to_epochs(reduction_params)
        markers = fc.keys()
        for marker in markers:
            df['marker'].append(marker)
            df['subject'].append(s)
            df['value'].append(fc[marker]) # one value per epoch
            df['avg_value'].append(np.mean(fc[marker])) # average of values of all epochs    
        logger.info('Done')
    df = pd.DataFrame(df)
    
    outname = op.join(
        outpath, 'ns_primate_markers_'+stim_cond+'.csv')


